@@ -6,9 +6,9 @@
 ### Maximum Likelihood inference for the two-component spatio-temporal intensity
 ### model described in Meyer et al (2012), DOI: 10.1111/j.1541-0420.2011.01684.x
 ###
-### Copyright (C) 2009-2017 Sebastian Meyer
-### $Revision: 1901 $
-### $Date: 2017-06-20 17:17:41 +0200 (Tue, 20. Jun 2017) $
+### Copyright (C) 2009-2019 Sebastian Meyer
+### $Revision: 2788 $
+### $Date: 2022-01-31 22:08:54 +0100 (Mon, 31. Jan 2022) $
 ################################################################################
 
 
@@ -41,9 +41,10 @@ twinstim <- function (
     epilinkinv <- switch(epilink, "log" = exp, "identity" = identity)
 
     ## Clean the model environment when exiting the function
+    if (model)
     on.exit(suppressWarnings(rm(cl, cumCIF, cumCIF.pb, data, doHessian,
         eventsData, finetune, neghess, fisherinfo, fit, fixed,
-        functions, globalEndemicIntercept, h.Intercept, inmfe, initpars,
+        functions, globalEndemicIntercept, inmfe, initpars,
         ll, negll, loglik, msgConvergence, msgNotConverged,
         mfe, mfhEvents, mfhGrid, model, my.na.action, na.action, namesOptimUser,
         namesOptimArgs, nlminbControl, nlminbRes, nlmObjective, nlmControl,
@@ -80,7 +81,7 @@ twinstim <- function (
         if (is.na(justBeforet0)) justBeforet0 <- length(data$stgrid$start)   # t0 was too big
         if (justBeforet0 == 0L) justBeforet0 <- 1L   # t0 was too small
         t0 <- data$stgrid$start[justBeforet0]
-        message("replaced 't0' by the value ", t0,
+        warning("replaced 't0' by the value ", t0,
                 " (must be a 'start' time of 'data$stgrid')")
     }
     if (!T %in% data$stgrid$stop) {
@@ -88,7 +89,7 @@ twinstim <- function (
         # if 'T' is beyond the time range covered by 'data$stgrid'
         if (is.na(justAfterT)) justAfterT <- length(data$stgrid$stop)   # T was too big
         T <- data$stgrid$stop[justAfterT]
-        message("replaced 'T' by the value ", T,
+        warning("replaced 'T' by the value ", T,
                 " (must be a 'stop' time of 'data$stgrid')")
     }
 
@@ -191,6 +192,7 @@ twinstim <- function (
     ### Generate model matrix
 
     mme <- model.matrix(epidemic, mfe)
+    xlevels_epidemic <- .getXlevels(epidemic, mfe)
     q <- ncol(mme)
     hase <- q > 0L
 
@@ -279,6 +281,7 @@ twinstim <- function (
                              # endemic = ~1 (see PR#14066)
                              drop.unused.levels = FALSE)
     mmhEvents <- model.matrix(endemic, mfhEvents)
+    xlevels_endemic <- .getXlevels(endemic, mfhEvents)
     # exclude intercept from endemic model matrix below, will be treated separately
     if (nbeta0 > 0) mmhEvents <- mmhEvents[,-1,drop=FALSE]
     #stopifnot(nrow(mmhEvents) == Nin)
@@ -335,7 +338,7 @@ twinstim <- function (
         if (nbeta0 == 1L) { # global intercept
             hEventsExpr <- call("+", quote(beta0), hEventsExpr)
         } else if (nbeta0 > 1L) { # type-specific intercept
-            hEventsExpr <- call("+", quote(beta0[eventTypes]), hEventsExpr)
+            hEventsExpr <- call("+", quote(beta0[eventTypes[includes]]), hEventsExpr)
         }
         if (!is.null(offsetEvents))
             hEventsExpr <- call("+", quote(offsetEvents), hEventsExpr)
@@ -687,12 +690,12 @@ twinstim <- function (
                 sEventsSum - unname(sInt)
             }) else if (nbeta0 > 1L) local({ # type-specific intercepts
                 ind <- sapply(seq_len(nTypes),
-                              function (type) eventTypes == type,
-                              simplify=TRUE, USE.NAMES=FALSE) # logical N x nTypes matrix
+                              function (type) eventTypes[includes] == type,
+                              simplify=TRUE, USE.NAMES=FALSE) # logical Nin x nTypes matrix
                 sEvents <- if (hase) {
                         ind * hEvents / lambdaEvents
                     } else ind
-                sEventsSum <- .colSums(sEvents, N, nTypes)
+                sEventsSum <- .colSums(sEvents, Nin, nTypes)
                 sInt <- exp(beta0) * .hIntTW(beta)
                 sEventsSum - unname(sInt)
             }) else numeric(0L) # i.e. nbeta0 == 0L
@@ -789,8 +792,8 @@ twinstim <- function (
         hScoreEvents <- if (hash) {
             scoreEvents_beta0 <- if (nbeta0 > 1L) local({ # type-specific intercepts
                 ind <- sapply(seq_len(nTypes),
-                              function (type) eventTypes == type,
-                              simplify=TRUE, USE.NAMES=FALSE) # logical N x nTypes matrix
+                              function (type) eventTypes[includes] == type,
+                              simplify=TRUE, USE.NAMES=FALSE) # logical Nin x nTypes matrix
                 if (hase) {
                     ind * hEvents / lambdaEvents
                 } else ind
@@ -916,7 +919,7 @@ twinstim <- function (
         body(ll) <- as.call(append(as.list(body(ll)),
             as.list(expression(
                 if (hassiafpars && !siaf$validpars(siafpars)) {
-                    if (optimArgs$control$trace > 0L)
+                    if (!isTRUE(optimArgs$control$trace == 0)) # default: NULL
                         cat("(invalid 'siafpars' in loglik)\n")
                     return(-Inf)
                 }
@@ -928,7 +931,7 @@ twinstim <- function (
         body(ll) <- as.call(append(as.list(body(ll)),
             as.list(expression(
                 if (hastiafpars && !tiaf$validpars(tiafpars)) {
-                    if (optimArgs$control$trace > 0L)
+                    if (!isTRUE(optimArgs$control$trace == 0)) # default: NULL
                         cat("(invalid 'tiafpars' in loglik)\n")
                     return(-Inf)
                 }
@@ -960,15 +963,35 @@ twinstim <- function (
 
     ### Check initial value for theta
 
-    if (is.null(optim.args[["par"]])) { # set naive defaults
-        h.Intercept <- if (nbeta0 > 0) rep.int(crudebeta0(
+    initpars <- rep(0, npars)
+    names(initpars) <- c(
+        if (nbeta0 > 1L) {
+            paste0("h.type",typeNames)
+        } else if (nbeta0 == 1L) "h.(Intercept)",
+        if (p > 0L) paste("h", colnames(mmhEvents), sep = "."),
+        if (hase) paste("e", colnames(mme), sep = "."),
+        if (hassiafpars) paste("e.siaf", seq_len(nsiafpars), sep="."),
+        if (hastiafpars) paste("e.tiaf", seq_len(ntiafpars), sep=".")
+    )
+
+    ## some naive defaults
+    if (nbeta0 > 0)
+        initpars[seq_len(nbeta0)] <- crudebeta0(
             nEvents = Nin,
             offset.mean = if (is.null(offsetGrid)) 0 else weighted.mean(offsetGrid, ds),
             W.area = sum(ds[gridBlocks==histIntervals[1,"BLOCK"]]),
             period = T-t0, nTypes = nTypes
-            ), nbeta0) else numeric(0L)
-        optim.args$par <- c(h.Intercept, rep.int(0, npars - nbeta0))
-    } else { # check validity of par-specification
+        )
+    if (hase && "e.(Intercept)" %in% names(initpars) && epilink == "log")
+        initpars["e.(Intercept)"] <- -9  # suitable value depends on [st]iafInt
+    if (hassiafpars && identical(body(siaf$f)[[2L]], quote(sds <- exp(pars)))) {
+        ## "detect" siaf.gaussian => use 10% of bbox diameter as initial sd
+        initpars[paste0("e.siaf.", seq_len(nsiafpars))] <-
+            round(log(0.1*sqrt(sum(apply(bbox(data$W), 1L, diff.default)^2))))
+    }
+
+    ## manual par-specification overrides these defaults
+    if (!is.null(optim.args[["par"]])) {
         if (!is.vector(optim.args$par, mode="numeric")) {
             stop("'optim.args$par' must be a numeric vector")
         }
@@ -977,27 +1000,23 @@ twinstim <- function (
                                 "length as the number of unknown parameters (%d)"),
                           length(optim.args$par), npars))
         }
+        initpars[] <- optim.args$par
     }
 
-    ## Set names for theta
-    names(optim.args$par) <- c(
-        if (nbeta0 > 1L) {
-            paste0("h.type",typeNames)
-        } else if (nbeta0 == 1L) "h.(Intercept)",
-        if (p > 0L) paste("h", colnames(mmhEvents), sep = "."),
-        if (hase) paste("e", colnames(mme), sep = "."),
-        if (hassiafpars) paste("e.siaf",1:nsiafpars,sep="."),
-        if (hastiafpars) paste("e.tiaf",1:ntiafpars,sep=".")
-    )
-
-    ## values in "start" overwrite initial values given by optim.args$par
+    ## values in "start" overwrite defaults and optim.args$par
     if (!is.null(start)) {
         start <- check_twinstim_start(start)
-        start <- start[names(start) %in% names(optim.args$par)]
-        optim.args$par[names(start)] <- start
+        start <- start[names(start) %in% names(initpars)]
+        initpars[names(start)] <- start
     }
 
-    initpars <- optim.args$par
+    ## warn if initial intercept is negative when the identity link is used
+    if (epilink == "identity" && "e.(Intercept)" %in% names(initpars) &&
+        initpars["e.(Intercept)"] < 0)
+        warning("identity link and negative start value for \"e.(Intercept)\"")
+
+    ## update optim.args$par
+    optim.args$par <- initpars
 
 
     ### Fixed parameters during optimization
@@ -1373,6 +1392,10 @@ twinstim <- function (
     fit$timeRange <- c(t0, T)           # for simulate.twinstim's defaults
     fit$formula <- list(endemic = endemic, epidemic = epidemic,
                         siaf = siaf, tiaf = tiaf)
+    fit["xlevels"] <- list(
+        if (length(xlevels_endemic) + length(xlevels_epidemic) > 0) {
+            list(endemic = xlevels_endemic, epidemic = xlevels_epidemic)
+        } else NULL)
     fit["control.siaf"] <- list(control.siaf)    # might be NULL
 
 
